from sqlalchemy.orm import Session

from jwtserver import crud, schemas
from jwtserver.core.config import settings
from jwtserver.db import base  # noqa: F401


def init_db(db: Session) -> None:
    # Tables should be created with Alembic migrations
    # But if you don't want to use migrations, create
    # the tables un-commenting the next line
    # Base.metadata.create_all(bind=engine)

    user = crud.user.get_by_email(db, email=settings.FIRST_SUPERUSER)
    if not user:
        user_in = schemas.UserCreate(
            email=settings.FIRST_SUPERUSER,
            password=settings.FIRST_SUPERUSER_PASSWORD,
            is_superuser=True,
        )
        user = crud.user.create(db, obj_in=user_in)  # noqa: F841



from typing import Any, List

from fastapi import APIRouter, Body, Depends, HTTPException
from fastapi.encoders import jsonable_encoder
from pydantic.networks import EmailStr
from sqlalchemy.orm import Session

from jwtserver import crud, models, schemas
from jwtserver.api import deps
from jwtserver.core.config import settings
from jwtserver.utils.utils import send_new_account_email

router = APIRouter()


@router.get("/", response_model=List[schemas.User])
def read_users(
    db: Session = Depends(deps.get_db),
    skip: int = 0,
    limit: int = 100,
    current_user: models.User = Depends(deps.get_current_active_superuser),
) -> Any:
    """
    Retrieve users.
    """
    users = crud.user.get_multi(db, skip=skip, limit=limit)
    return users


@router.post("/", response_model=schemas.User)
def create_user(
    *,
    db: Session = Depends(deps.get_db),
    user_in: schemas.UserCreate,
    current_user: models.User = Depends(deps.get_current_active_superuser),
) -> Any:
    """
    Create new user.
    """
    user = crud.user.get_by_email(db, email=user_in.email)
    if user:
        raise HTTPException(
            status_code=400,
            detail="The user with this username already exists in the system.",
        )
    user = crud.user.create(db, obj_in=user_in)
    if settings.EMAILS_ENABLED and user_in.email:
        send_new_account_email(
            email_to=user_in.email, username=user_in.email, password=user_in.password
        )
    return user


@router.put("/me", response_model=schemas.User)
def update_user_me(
    *,
    db: Session = Depends(deps.get_db),
    password: str = Body(None),
    full_name: str = Body(None),
    email: EmailStr = Body(None),
    current_user: models.User = Depends(deps.get_current_active_user),
) -> Any:
    """
    Update own user.
    """
    current_user_data = jsonable_encoder(current_user)
    user_in = schemas.UserUpdate(**current_user_data)
    if password is not None:
        user_in.password = password
    if full_name is not None:
        user_in.full_name = full_name
    if email is not None:
        user_in.email = email
    user = crud.user.update(db, db_obj=current_user, obj_in=user_in)
    return user


@router.get("/me", response_model=schemas.User)
def read_user_me(
    db: Session = Depends(deps.get_db),
    current_user: models.User = Depends(deps.get_current_active_user),
) -> Any:
    """
    Get current user.
    """
    return current_user


@router.post("/open", response_model=schemas.User)
def create_user_open(
    *,
    db: Session = Depends(deps.get_db),
    password: str = Body(...),
    email: EmailStr = Body(...),
    full_name: str = Body(None),
) -> Any:
    """
    Create new user without the need to be logged in.
    """
    if not settings.USERS_OPEN_REGISTRATION:
        raise HTTPException(
            status_code=403,
            detail="Open user registration is forbidden on this server",
        )
    user = crud.user.get_by_email(db, email=email)
    if user:
        raise HTTPException(
            status_code=400,
            detail="The user with this username already exists in the system",
        )
    user_in = schemas.UserCreate(password=password, email=email, full_name=full_name)
    user = crud.user.create(db, obj_in=user_in)
    return user


@router.get("/{user_id}", response_model=schemas.User)
def read_user_by_id(
    user_id: int,
    current_user: models.User = Depends(deps.get_current_active_user),
    db: Session = Depends(deps.get_db),
) -> Any:
    """
    Get a specific user by id.
    """
    user = crud.user.get(db, id=user_id)
    if user == current_user:
        return user
    if not crud.user.is_superuser(current_user):
        raise HTTPException(
            status_code=400, detail="The user doesn't have enough privileges"
        )
    return user


@router.put("/{user_id}", response_model=schemas.User)
def update_user(
    *,
    db: Session = Depends(deps.get_db),
    user_id: int,
    user_in: schemas.UserUpdate,
    current_user: models.User = Depends(deps.get_current_active_superuser),
) -> Any:
    """
    Update a user.
    """
    user = crud.user.get(db, id=user_id)
    if not user:
        raise HTTPException(
            status_code=404,
            detail="The user with this username does not exist in the system",
        )
    user = crud.user.update(db, db_obj=user, obj_in=user_in)
    return user



from typing import Any, List

from fastapi import APIRouter, Depends, HTTPException
from sqlalchemy.orm import Session

from jwtserver import crud, models, schemas
from jwtserver.api import deps

router = APIRouter()


@router.get("/", response_model=List[schemas.Item])
def read_items(
    db: Session = Depends(deps.get_db),
    skip: int = 0,
    limit: int = 100,
    current_user: models.User = Depends(deps.get_current_active_user),
) -> Any:
    """
    Retrieve items.
    """
    if crud.user.is_superuser(current_user):
        items = crud.item.get_multi(db, skip=skip, limit=limit)
    else:
        items = crud.item.get_multi_by_owner(
            db=db, owner_id=current_user.id, skip=skip, limit=limit
        )
    return items


@router.post("/", response_model=schemas.Item)
def create_item(
    *,
    db: Session = Depends(deps.get_db),
    item_in: schemas.ItemCreate,
    current_user: models.User = Depends(deps.get_current_active_user),
) -> Any:
    """
    Create new item.
    """
    item = crud.item.create_with_owner(db=db, obj_in=item_in, owner_id=current_user.id)
    return item


@router.put("/{id}", response_model=schemas.Item)
def update_item(
    *,
    db: Session = Depends(deps.get_db),
    id: int,
    item_in: schemas.ItemUpdate,
    current_user: models.User = Depends(deps.get_current_active_user),
) -> Any:
    """
    Update an item.
    """
    item = crud.item.get(db=db, id=id)
    if not item:
        raise HTTPException(status_code=404, detail="Item not found")
    if not crud.user.is_superuser(current_user) and (item.owner_id != current_user.id):
        raise HTTPException(status_code=400, detail="Not enough permissions")
    item = crud.item.update(db=db, db_obj=item, obj_in=item_in)
    return item


@router.get("/{id}", response_model=schemas.Item)
def read_item(
    *,
    db: Session = Depends(deps.get_db),
    id: int,
    current_user: models.User = Depends(deps.get_current_active_user),
) -> Any:
    """
    Get item by ID.
    """
    item = crud.item.get(db=db, id=id)
    if not item:
        raise HTTPException(status_code=404, detail="Item not found")
    if not crud.user.is_superuser(current_user) and (item.owner_id != current_user.id):
        raise HTTPException(status_code=400, detail="Not enough permissions")
    return item


@router.delete("/{id}", response_model=schemas.Item)
def delete_item(
    *,
    db: Session = Depends(deps.get_db),
    id: int,
    current_user: models.User = Depends(deps.get_current_active_user),
) -> Any:
    """
    Delete an item.
    """
    item = crud.item.get(db=db, id=id)
    if not item:
        raise HTTPException(status_code=404, detail="Item not found")
    if not crud.user.is_superuser(current_user) and (item.owner_id != current_user.id):
        raise HTTPException(status_code=400, detail="Not enough permissions")
    item = crud.item.remove(db=db, id=id)
    return item



from typing import Any, Dict, Optional, Union

from sqlalchemy.orm import Session

from jwtserver.core.security import get_password_hash, verify_password
from jwtserver.crud.base import CRUDBase
from jwtserver.models.user import User
from jwtserver.schemas.user import UserCreate, UserUpdate


class CRUDUser(CRUDBase[User, UserCreate, UserUpdate]):
    def get_by_email(self, db: Session, *, email: str) -> Optional[User]:
        return db.query(User).filter(User.email == email).first()

    def get_by_telephone(self, db: Session, *, telephone: str) -> Optional[User]:
        return db.query(User).filter(User.telephone == telephone).first()

    def create(self, db: Session, *, obj_in: UserCreate) -> User:
        db_obj = User(
            email=obj_in.email,
            hashed_password=get_password_hash(obj_in.password),
            full_name=obj_in.full_name,
            is_superuser=obj_in.is_superuser,
        )
        db.add(db_obj)
        db.commit()
        db.refresh(db_obj)
        return db_obj

    def update(
        self, db: Session, *, db_obj: User, obj_in: Union[UserUpdate, Dict[str, Any]]
    ) -> User:
        if isinstance(obj_in, dict):
            update_data = obj_in
        else:
            update_data = obj_in.dict(exclude_unset=True)
        if update_data["password"]:
            hashed_password = get_password_hash(update_data["password"])
            del update_data["password"]
            update_data["hashed_password"] = hashed_password
        return super().update(db, db_obj=db_obj, obj_in=update_data)

    def authenticate(self, db: Session, *, email: str, password: str) -> Optional[User]:
        user = self.get_by_email(db, email=email)
        if not user:
            return None
        if not verify_password(password, user.hashed_password):
            return None
        return user

    def is_active(self, user: User) -> bool:
        return user.is_active

    def is_superuser(self, user: User) -> bool:
        return user.is_superuser


user = CRUDUser(User)



from secrets import token_hex

from fastapi import Body, Depends, HTTPException
from jwtserver.api.api_v1.endpoints.reg.routers import router_api_v1_reg
from jwtserver.api.api_v1.endpoints.reg.types_model import CheckCodeResponseModel
from redis import Redis
from starlette import status

from jwtserver.old_dependencies.init_redis import redis_conn
from jwtserver.recaptcha.google import RecaptchaGoogleV3


# TODO: rewrite and add routes in API V1
@router_api_v1_reg.post(
    "/check_code",
    description="User authorization by login and password",
    # response_description=response_description,
    response_model=CheckCodeResponseModel,
)
async def check_code(
    # redis: Redis,
    telephone: str = Body(...),
    code: int = Body(...),
    redis: Redis = Depends(redis_conn),
    recaptcha: RecaptchaGoogleV3 = Depends(RecaptchaGoogleV3),
):
    """Checking the code from SMS or Call
    :param str telephone: Telephone number in international format
    :param int code: 4 digit verification code
    :param redis: Redis client
    :param recaptcha: Validate Google recaptcha_v3.md v3 [return True or HTTPException]
    :return: one-time token for registration
    """
    await recaptcha.set_action_name("SignUpPage/CheckCode").greenlight()

    code_method = redis.get(telephone)
    if code_method:
        from_redis_code, method = code_method.split(":")
        if int(from_redis_code) == code:
            reg_token = token_hex(16)
            redis.set(f"{telephone}_reg_token", reg_token, 60 * 60)
            return {"reg_token": reg_token}

        raise HTTPException(
            status_code=status.HTTP_400_BAD_REQUEST,
            detail="Неверный код",
        )
    raise HTTPException(
        status_code=status.HTTP_400_BAD_REQUEST,
        detail="Нужно запросить новый код ",
    )



from typing import TYPE_CHECKING

from sqlalchemy import Column, ForeignKey, Integer, String
from sqlalchemy.orm import relationship

from jwtserver.db.base_class import Base

if TYPE_CHECKING:
    from .user import User  # noqa: F401


class Item(Base):
    id = Column(Integer, primary_key=True, index=True)
    title = Column(String, index=True)
    description = Column(String, index=True)
    owner_id = Column(Integer, ForeignKey("user.id"))
    owner = relationship("User", back_populates="items")



from typing import Any

from fastapi import APIRouter, Depends
from pydantic.networks import EmailStr

from jwtserver import models, schemas
from jwtserver.api import deps
from jwtserver.core.celery_app import celery_jwtserver
from jwtserver.utils.utils import send_test_email

router = APIRouter()


@router.post("/test-celery/", response_model=schemas.Msg, status_code=201)
def test_celery(
    msg: schemas.Msg,
    current_user: models.User = Depends(deps.get_current_active_superuser),
) -> Any:
    """
    Test Celery worker.
    """
    celery_jwtserver.send_task("app.worker.test_celery", args=[msg.msg])
    return {"msg": "Word received"}


@router.post("/test-email/", response_model=schemas.Msg, status_code=201)
def test_email(
    email_to: EmailStr,
    current_user: models.User = Depends(deps.get_current_active_superuser),
) -> Any:
    """
    Test emails.
    """
    send_test_email(email_to=email_to)
    return {"msg": "Test email sent"}



import asyncio
import os
from typing import AsyncGenerator, Any

import pytest
import redis
from asgi_lifespan import LifespanManager
from fastapi import FastAPI
from httpx import AsyncClient
from redis import Redis

from jwtserver import create_app
from jwtserver.old_dependencies.init_redis import redis_conn
from jwtserver.old_dependencies.session_db import async_db_session
from jwtserver.tests.depends import override_async_db_session, override_redis_conn

os.environ["DEBUG"] = "False"


@pytest.fixture(scope="module")
def event_loop():
    return asyncio.get_event_loop()


@pytest.fixture(scope="module", autouse=True)
async def flushall():
    r = redis.from_url("redis://:@localhost:6380/1", decode_responses=True)
    r.flushall()
    r.close()


@pytest.fixture(scope="module")
def app() -> FastAPI:
    # settings = ApplicationSettings()
    # director = Director(DevelopmentApplicationBuilder(settings=settings))
    _app = create_app(lvl_logging="CRITICAL")
    _app.dependency_overrides[async_db_session] = override_async_db_session
    _app.dependency_overrides[redis_conn] = override_redis_conn
    return _app


@pytest.fixture(scope="module")
async def initialized_app(app: FastAPI) -> AsyncGenerator[FastAPI, Any]:
    async with LifespanManager(app):
        yield app


@pytest.fixture(scope="module")
async def client(initialized_app: FastAPI) -> AsyncGenerator[AsyncClient, Any]:
    async with AsyncClient(
        app=initialized_app,
        base_url="http://test",
        headers={"Content-Type": "application/json", "accept": "application/json"},
    ) as client:  # type: AsyncClient
        yield client


@pytest.fixture(scope="module")
def authorized_client(client: AsyncClient) -> AsyncClient:
    return client


@pytest.fixture(scope="module")
def redis_client123():
    pass


@pytest.fixture(scope="module")
async def redis_client() -> Redis.client:
    r = redis.from_url("redis://:@localhost:6380/1", decode_responses=True)
    _client = r.client()
    try:
        return _client
    finally:
        _client.close()



import uuid as _uuid
from sqlalchemy import Boolean, Column, Integer, String
from sqlalchemy_utils.types.phone_number import PhoneNumberType
from jwtserver.database import Base

from sqlalchemy.types import TypeDecorator, CHAR
from sqlalchemy.dialects.postgresql import UUID
import uuid


class GUID(TypeDecorator):
    """Platform-independent GUID type.

    Uses PostgreSQL's UUID type, otherwise uses
    CHAR(32), storing as stringified hex values.

    """
    impl = CHAR
    cache_ok = True

    def load_dialect_impl(self, dialect):
        if dialect.name == 'postgresql':
            return dialect.type_descriptor(UUID())
        else:
            return dialect.type_descriptor(CHAR(32))

    def process_bind_param(self, value, dialect):
        if value is None:
            return value
        elif dialect.name == 'postgresql':
            return str(value)
        else:
            if not isinstance(value, uuid.UUID):
                return "%.32x" % uuid.UUID(value).int
            else:
                # hexstring
                return "%.32x" % value.int

    def process_result_value(self, value, dialect):
        if value is None:
            return value
        else:
            if not isinstance(value, uuid.UUID):
                value = uuid.UUID(value)
            return value


class User(Base):
    __tablename__ = "users"

    uuid = Column(GUID, primary_key=True, default=_uuid.uuid4)
    telephone = Column(PhoneNumberType())
    password = Column(String)
    is_active = Column(Boolean, default=True)

    def __repr__(self):
        return "<User('%s','%s')>" % (self.uuid, self.telephone)


class RefreshToken(Base):
    __tablename__ = "refresh_token"
    id = Column(Integer, primary_key=True, index=True)
    token = Column(String, unique=True, index=True)



from typing import Generator

from fastapi import Depends, HTTPException, status, Body, Request
from fastapi.security import OAuth2PasswordBearer
from jose import jwt
from pydantic import ValidationError
from sqlalchemy.orm import Session

from jwtserver import crud, models, schemas
from jwtserver.core import security
from jwtserver.core.config import settings
from jwtserver.db.session import SessionLocal
from jwtserver.recaptcha.google import RecaptchaGoogleV3

reusable_oauth2 = OAuth2PasswordBearer(
    tokenUrl=f"{settings.API_V1_STR}/login/access-token"
)


def get_db() -> Generator:
    try:
        db = SessionLocal()
        yield db
    finally:
        db.close()


def get_current_user(
    db: Session = Depends(get_db), token: str = Depends(reusable_oauth2)
) -> models.User:
    try:
        payload = jwt.decode(
            token, settings.SECRET_KEY, algorithms=[security.ALGORITHM]
        )
        token_data = schemas.TokenPayload(**payload)
    except (jwt.JWTError, ValidationError):
        raise HTTPException(
            status_code=status.HTTP_403_FORBIDDEN,
            detail="Could not validate credentials",
        )
    user = crud.user.get(db, id=token_data.sub)
    if not user:
        raise HTTPException(status_code=404, detail="User not found")
    return user


def get_current_active_user(
    current_user: models.User = Depends(get_current_user),
) -> models.User:
    if not crud.user.is_active(current_user):
        raise HTTPException(status_code=400, detail="Inactive user")
    return current_user


def get_current_active_superuser(
    current_user: models.User = Depends(get_current_user),
) -> models.User:
    if not crud.user.is_superuser(current_user):
        raise HTTPException(
            status_code=400, detail="The user doesn't have enough privileges"
        )
    return current_user


async def recaptcha_google_v3_action(request: Request, recaptcha=Body(...)):
    # if self.environment == "tests":
    #     self.r_json = {
    #         "success": self.recaptcha_token.split(":")[0],
    #         "action": self.recaptcha_token.split(":")[1],
    #         "score": float(self.recaptcha_token.split(":")[2]),
    #     }
    #     return self
    recaptcha = RecaptchaGoogleV3(
        _config=settings.google.recaptcha,
        recaptcha_token=recaptcha,
    )
    await recaptcha.set_action_name(request.url.path).greenlight()

    request.state.recaptcha = recaptcha



import redis
from sqlalchemy import create_engine
from sqlalchemy.orm import sessionmaker

from jwtserver import models

engine = create_engine(
    "postgresql+psycopg2://jwtserver:jwtserver-password@localhost:5433/jwtserver-tests"
)
# async_engine = create_async_engine(
#     "postgresql+asyncpg://jwtserver:jwtserver-password@localhost:5433/jwtserver-tests"
# )
# TestingSessionLocal = sessionmaker(
#     async_engine, expire_on_commit=False, class_=AsyncSession
# )
TestingSessionLocal = sessionmaker(engine)


def create_pool_redis():
    _pool = redis.ConnectionPool.from_url(
        "redis://:@localhost:6380/1",
        max_connections=10,
        decode_responses=True,
    )
    return redis.Redis(connection_pool=_pool)


models.Base.metadata.drop_all(bind=engine)
models.Base.metadata.create_all(bind=engine)
# models.Base.metadata.drop_all(bind=sync_engine)
# models.Base.metadata.create_all(bind=sync_engine)


def override_async_db_session():
    """Databases pool fabric connection, auto close connection"""
    try:
        db = TestingSessionLocal()
        yield db
    finally:
        db.close()


def override_redis_conn():
    try:
        r = create_pool_redis().client()
        yield r
    finally:
        r.close()



import logging
import sys
import uvicorn
from loguru import logger


def enable_logger(sink=sys.stderr, level='WARNING'):
    logging.basicConfig(level=logging.DEBUG)
    logger.configure(handlers=[{'sink': sink, 'level': level}])
    logger.enable('aria2p')


def dev(host='localhost', port=5000, log_level='info'):
    enable_logger(level=log_level.upper())
    uvicorn.run('jwtserver.app:app', host=host, port=port, log_level=log_level)



import redis

from jwtserver.core.config import settings


def create_pool_redis():
    pool = redis.ConnectionPool.from_url(
        settings.redis.redis_dsn,
        max_connections=settings.redis.max_connections,
        decode_responses=True,
    )
    return redis.Redis(connection_pool=pool)


def redis_conn():
    r = create_pool_redis().client()
    try:
        yield r
    finally:
        r.close()



from fastapi import Body, Depends, HTTPException
from jwtserver.api.api_v1.endpoints.reg.routers import router_api_v1_reg
from jwtserver.api.api_v1.endpoints.reg.types_model import RespSendCodeModel
from jwtserver.settings import Settings, get_settings
from redis import Redis
from starlette import status

from jwtserver.internal.SMSCRules import SMSRules
from jwtserver.old_dependencies.init_redis import redis_conn


# TODO: rewrite and add routes in API V1
@router_api_v1_reg.post(
    "/send_code",
    response_model=RespSendCodeModel,
    description="Sending a code through a call or SMS",
    name="reg:send_code",
)
async def send_code(
    telephone: str = Body(...),
    settings: Settings = Depends(get_settings),
    redis: Redis = Depends(redis_conn),
):
    sms_rules = SMSRules(settings, redis)
    result = await sms_rules.send_code(telephone)

    if result.error:
        raise HTTPException(
            status_code=status.HTTP_409_CONFLICT,
            detail={
                "error": result.error,
                "block_time": result.error.details.block_time,
            },
        )

    return {
        "send": True,
        "block_time": result.is_sent.block_time.timestamp(),
        "method": result.is_sent.method,
    }



from multiprocessing import cpu_count

from jwtserver.utils.config import load_config

config = load_config().server


def max_workers():
    return cpu_count()


bind = f"{config.host}:{config.port}"
max_requests = 1000
worker_class = "uvicorn.workers.UvicornWorker"
workers = max_workers()
threads = max_workers() * 2
reload = True
name = "troll"



from datetime import datetime, timedelta
from random import randint
from typing import Callable, Literal

import phonenumbers as phnbrs
from phonenumbers import PhoneNumber, NumberParseException
from pydantic import BaseModel, validator
from redis import Redis

from jwtserver.settings import Settings


class SMSFabric(BaseModel):
    calling: Callable
    send_sms: Callable


class Details(BaseModel):
    block_time: str | None


class ErrorModel(BaseModel):
    text: str
    details: Details | None


class SendDetailsModel(BaseModel):
    method: Literal["call", "sms"]
    block_time: datetime
    attempts: int  # попытки
    ip: str | None

    @validator("block_time", pre=True, always=True)
    def set_block_time_now(cls, v):
        return datetime.fromtimestamp(float(v)) or None


class SendCodeReturnModel(BaseModel):
    is_sent: SendDetailsModel | None = None
    error: ErrorModel | None = None


class SMSRules:
    def __init__(self, settings: Settings, redis):
        self.sms_fabric = settings.sms.sms_fabric(settings.sms)
        self.redis: Redis = redis
        self.ignore_attempts = settings.sms.ignore_attempts
        self.try_call = settings.sms.try_call
        self.time_sms = settings.sms.time_sms
        self.time_calling = settings.sms.time_call
        self.try_sms = settings.sms.try_sms
        self.block_time_minutes = settings.sms.block_time_minutes
        self.general_try_count = self.try_call + self.try_sms

    async def send_code(self, tel: str) -> SendCodeReturnModel:
        """
        Valid phone number check.
        Checking for verification code sending restrictions
        :param tel: Phone number in international format.
        :return: SendCodeReturnModel.
        """
        if not self.tel_is_valid(tel):
            error = ErrorModel(text=f"Wrong number: {tel}.")
            return SendCodeReturnModel(is_sent=None, error=error)

        if await self.limit_is_over(tel):
            error = ErrorModel(text=f"Request limit exceeded: {tel}.")
            return SendCodeReturnModel(is_sent=None, error=error)

        last_code_send = await self.code_is_send(tel)

        if not last_code_send:
            return await self.calling(tel)

        return await self.send_sms(tel)

    async def calling(self, tel) -> SendCodeReturnModel:
        code, error = await self.sms_fabric.calling(tel)
        if error:
            return SendCodeReturnModel(is_sent=None, error=error)
        block_time = datetime.now() + timedelta(seconds=self.try_call)
        mapping = {
            "method": "call",
            "block_time": block_time.timestamp(),
            "attempts": 1,
        }

        self.redis.hset(tel, mapping=mapping)
        return SendCodeReturnModel(is_sent=SendDetailsModel(**mapping))

    async def send_sms(self, tel) -> SendCodeReturnModel:
        code = "{:0>4d}".format(randint(0, 9999))
        sent, error = await self.sms_fabric.send_sms(tel, code)
        if error:
            return SendCodeReturnModel(is_sent=None, error=error)
        mapping = {"method": "sms"}
        self.redis.hset(tel, mapping={"method": "sms"})
        return SendCodeReturnModel(is_sent=SendDetailsModel(**mapping))

    def tel_is_valid(self, tel) -> bool:
        result_parse = self.parse_tel(tel)
        if result_parse:
            return phnbrs.is_valid_number(self.parse_tel(tel))
        return False

    @staticmethod
    def parse_tel(tel) -> PhoneNumber | None:
        try:
            return phnbrs.parse(tel, None)
        except NumberParseException:
            return None

    async def limit_is_over(self, tel):
        if self.ignore_attempts:
            self.redis.delete(f"{tel}")
        return self.general_try_count <= await self.get_try_count(tel)

    def code_is_valid(self, code, tel):
        pass

    async def code_is_send(self, tel) -> str | None:
        return self.redis.hexists(tel, "method")

    async def get_try_count(self, tel):
        return self.redis.hget(tel, "attempts") or 0

    def get_block_time(self, tel):
        pass

    #
    # async def get_ttl_try_count(self, tel):
    #     return await self.redis.ttl(f'{tel}_try_count')
    #
    # async def try_count_incr(self, tel):
    #     return await self.redis.incr(f'{tel}_try_count')
    #
    # async def try_count_decr(self, tel):
    #     return await self.redis.decr(f'{tel}_try_count')



from typing import Literal

import httpx
from fastapi import HTTPException
from loguru import logger
from pydantic import BaseModel
from starlette import status


class RecaptchaConfig(BaseModel):
    secret_key: str
    score: float
    for_tests: bool


class RecaptchaGoogleV3:

    """Recaptcha v3
    :raises HTTPException:
    https://www.google.com/recaptcha/admin/create"""

    def __init__(self, _config, recaptcha_token):
        self.config: RecaptchaConfig = _config
        self.action_name = None
        self.success = False
        self.action_valid = False
        self.r_json = None
        self.recaptcha_token = recaptcha_token

    def set_action_name(self, name) -> "RecaptchaGoogleV3":
        """Set google action name"""
        self.action_name = name
        return self

    async def greenlight(self) -> Literal[True]:
        """If the recaptcha is ok, then the light is green.
        our minimum checks
        :return: True or raise HTTPException
        """

        def bad_request(detail: str):
            raise HTTPException(
                status_code=status.HTTP_400_BAD_REQUEST,
                detail=detail,
            )

        await self.check()
        if not self.r_json["success"]:
            logger.critical("invalid reCAPTCHA token")
            bad_request("server error: invalid reCAPTCHA token for your site")

        if not self.r_json["action"] == self.action_name:
            bad_request("hm... u r hacker?")

        if self.r_json["score"] <= self.config.score:
            bad_request("hm... u r bot?")

        return True

    async def check(self) -> "RecaptchaGoogleV3":
        """send post and save response json to self.r_json"""

        data = {
            "secret": self.config.secret_key,
            "response": self.recaptcha_token,
        }
        async with httpx.AsyncClient() as client:
            r: httpx.Response = await client.post(
                "https://www.google.com/recaptcha/api/siteverify", data=data
            )
            self.r_json = r.json()
        return self



from pathlib import Path

from alembic import command
from alembic.config import Config

ROOT_PATH = Path(__file__).parent.parent
ALEMBIC_CFG = Config(ROOT_PATH / "alembic.ini")


def current(verbose=False):
    command.current(ALEMBIC_CFG, verbose=verbose)


def upgrade(revision="head"):
    command.upgrade(ALEMBIC_CFG, revision)


def downgrade(revision):
    command.downgrade(ALEMBIC_CFG, revision)



from typing import Optional

from pydantic import BaseModel, EmailStr


# Shared properties
class UserBase(BaseModel):
    email: Optional[EmailStr] = None
    is_active: Optional[bool] = True
    is_superuser: bool = False
    full_name: Optional[str] = None


# Properties to receive via API on creation
class UserCreate(UserBase):
    email: EmailStr
    password: str


# Properties to receive via API on update
class UserUpdate(UserBase):
    password: Optional[str] = None


class UserInDBBase(UserBase):
    id: Optional[int] = None

    class Config:
        orm_mode = True


# Additional properties to return via API
class User(UserInDBBase):
    pass


# Additional properties stored in DB
class UserInDB(UserInDBBase):
    hashed_password: str



from random import randint

import httpx
from pydantic import BaseModel, PyObject

__all__ = ["SMSC", "SMSModel"]


def string_to_dict(text: str):
    return {
        k.strip(): v.strip() for k, v in [b.split(" - ", 1) for b in text.split(",")]
    }


class SMSModel(BaseModel):
    debug: bool = True  # imitation of sending SMS messages.
    ignore_attempts: bool = False
    try_call: int = 2
    try_sms: int = 2
    block_time_minutes: int = 180
    sms_fabric: PyObject = "jwtserver.utils.SMSC.SMSC"
    login: str = None
    password: str = None
    time_sms: int = 120
    time_call: int = 90


class SMSC:
    url = "https://smsc.ru/sys/send.php?"

    def __init__(self, _config):
        self.config: SMSModel = _config
        self.param_url = {"login": self.config.login, "psw": self.config.password}

    async def calling(self, telephone) -> tuple[str | None, str | None]:
        param_url = {
            **self.param_url,
            **{"phones": telephone, "mes": "code", "call": 1},
        }

        if self.config.debug:
            code = "{:0>4d}".format(randint(0, 9999))
        else:
            async with httpx.AsyncClient() as client:
                resp: httpx.Response = await client.get(self.url, param_url)
                if resp.status_code == httpx.codes.OK:
                    text = resp.text
                    if "ERROR" in text:
                        return None, "Wait1min"
                    data = string_to_dict(text)
                else:
                    return None, "response status not 200 OK"

            code = data["CODE"][-4:]
        return code, None

    async def send_sms(self, telephone, code) -> [bool, str | None]:
        param_url = {**self.param_url, **{"phones": telephone, "mes": code}}
        if self.config.debug:
            return True, None

        async with httpx.AsyncClient() as client:
            resp: httpx.Response = await client.get(self.url, param_url)
            if resp.status_code == httpx.codes.OK:
                text = resp.text
                if "ERROR" in text:
                    return False, "Wait1min"
            else:
                return False, "response status not 200 OK"
            return True, None



"""initial

Revision ID: 98f3d19d2c0d
Revises:
Create Date: 2023-03-18 15:38:10.466687

"""
import sqlalchemy as sa
from alembic import op

# revision identifiers, used by Alembic.
revision = "98f3d19d2c0d"
down_revision = None
branch_labels = None
depends_on = None


def upgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.create_table(
        "user",
        sa.Column("id", sa.Integer(), nullable=False),
        sa.Column("full_name", sa.String(), nullable=True),
        sa.Column("email", sa.String(), nullable=False),
        sa.Column("telephone", sa.String(), nullable=False),
        sa.Column("hashed_password", sa.String(), nullable=False),
        sa.Column("is_active", sa.Boolean(), nullable=True),
        sa.Column("is_superuser", sa.Boolean(), nullable=True),
        sa.PrimaryKeyConstraint("id"),
    )
    op.create_index(op.f("ix_user_email"), "user", ["email"], unique=True)
    op.create_index(op.f("ix_user_full_name"), "user", ["full_name"], unique=False)
    op.create_index(op.f("ix_user_id"), "user", ["id"], unique=False)
    op.create_index(op.f("ix_user_telephone"), "user", ["telephone"], unique=True)
    op.create_table(
        "item",
        sa.Column("id", sa.Integer(), nullable=False),
        sa.Column("title", sa.String(), nullable=True),
        sa.Column("description", sa.String(), nullable=True),
        sa.Column("owner_id", sa.Integer(), nullable=True),
        sa.ForeignKeyConstraint(
            ["owner_id"],
            ["user.id"],
        ),
        sa.PrimaryKeyConstraint("id"),
    )
    op.create_index(op.f("ix_item_description"), "item", ["description"], unique=False)
    op.create_index(op.f("ix_item_id"), "item", ["id"], unique=False)
    op.create_index(op.f("ix_item_title"), "item", ["title"], unique=False)
    # ### end Alembic commands ###


def downgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_index(op.f("ix_item_title"), table_name="item")
    op.drop_index(op.f("ix_item_id"), table_name="item")
    op.drop_index(op.f("ix_item_description"), table_name="item")
    op.drop_table("item")
    op.drop_index(op.f("ix_user_telephone"), table_name="user")
    op.drop_index(op.f("ix_user_id"), table_name="user")
    op.drop_index(op.f("ix_user_full_name"), table_name="user")
    op.drop_index(op.f("ix_user_email"), table_name="user")
    op.drop_table("user")
    # ### end Alembic commands ###



import hashlib
from base64 import b64decode
from datetime import datetime, timedelta
from json import loads
from typing import Literal

from fastapi import HTTPException
from fastapi.param_functions import Optional
from jose import JWTError, jwt
from loguru import logger
from pydantic import BaseModel
from starlette import status

from jwtserver.core.config import TokenModel


class Data(BaseModel):
    access_token: Optional[str] = None
    refresh_token: Optional[str] = None


def secret(string, base_sol: str, temporary_sol: str = ""):
    sol = f"{base_sol[:50]}{temporary_sol}){base_sol[:50]}"
    return hashlib.pbkdf2_hmac(
        "sha256", string.encode("ascii"), sol.encode("ascii"), 100000
    ).hex()


class TokenProcessor:
    def __init__(
        self,
        _config: TokenModel,
        refresh_token: str = None,
        access_token: str = None,
    ):
        self.config = _config
        self.access = access_token
        self.new_access = access_token
        self.refresh = refresh_token
        self.new_refresh = refresh_token

    def payload_token_untested(self, token_type: Literal["access", "refresh"]):
        """Token untested payload
        :param str token_type:
        :return dict data: token payload
        """
        return loads(b64decode(getattr(self, token_type).split(".", 2)[1] + "=="))

    def payload_token(self, token_type: Literal["access", "refresh"]):
        """Token tested payload
        :param str token_type:
        :return dict data: token payload
        :raises JWTError: If the signature is invalid in any way
        :raises ExpiredSignatureError: If the signature has expired
        :raises JWTClaimsError: If any claim is invalid in any way
        """
        try:
            return jwt.decode(
                getattr(self, token_type),
                self.config.secret_key,
                algorithms=[self.config.algorithm],
            )
        except JWTError:
            raise HTTPException(
                status_code=status.HTTP_400_BAD_REQUEST,
                detail="Bad JWT token",
                headers={"WWW-Authenticate": "JSv1"},
            )

    def create_pair_tokens(self, uuid):
        """Create two JWT tokens with a shared secret at the same time
        :return tuple: [access_token, refresh_token]
        """
        access_time = timedelta(minutes=self.config.access_expire_time)
        refresh_time = timedelta(minutes=self.config.refresh_expire_time)

        datetime_now = datetime.now()
        temporary_sol_access = (datetime_now + access_time).timestamp()
        temporary_sol_refresh = (datetime_now + refresh_time).timestamp()
        payload_access = {
            "uuid": uuid,
            "secret": secret(
                uuid,
                base_sol=self.config.base_sol,
                temporary_sol=str(temporary_sol_access),
            )[:32],
            "exp": temporary_sol_access,
        }

        payload_refresh = {
            "secret": secret(
                uuid,
                base_sol=self.config.base_sol,
                temporary_sol=str(temporary_sol_refresh),
            )[32:],
            "exp": temporary_sol_refresh,
        }

        access_jwt = jwt.encode(
            payload_access, self.config.secret_key, algorithm=self.config.algorithm
        )

        refresh_jwt = jwt.encode(
            payload_refresh, self.config.secret_key, algorithm=self.config.algorithm
        )

        logger.info(f"create new {access_jwt=}")
        logger.info(f"create mew {refresh_jwt=}")
        return access_jwt, refresh_jwt



from sqlalchemy import create_engine
from sqlalchemy.orm import sessionmaker

from jwtserver.core.config import settings

engine = create_engine(settings.postgres.dsn, pool_pre_ping=True)
SessionLocal = sessionmaker(autocommit=False, autoflush=False, bind=engine)



from datetime import timedelta
from typing import Any

from fastapi import APIRouter, Body, Depends, HTTPException
from fastapi.security import OAuth2PasswordRequestForm
from sqlalchemy.orm import Session

from jwtserver import crud, models, schemas
from jwtserver.api import deps
from jwtserver.core import security
from jwtserver.core.config import settings
from jwtserver.core.security import get_password_hash
from jwtserver.utils.utils import (
    generate_password_reset_token,
    send_reset_password_email,
    verify_password_reset_token,
    telephone_validate,
)

router = APIRouter()


@router.post("/login/access-token", response_model=schemas.Token)
def login_access_token(
    db: Session = Depends(deps.get_db),
    form_data: OAuth2PasswordRequestForm = Depends(),
) -> Any:
    """
    OAuth2 compatible token login, get an access token for future requests
    """
    user = crud.user.authenticate(
        db, email=form_data.username, password=form_data.password
    )
    if not user:
        raise HTTPException(status_code=400, detail="Incorrect email or password")
    elif not crud.user.is_active(user):
        raise HTTPException(status_code=400, detail="Inactive user")
    access_token_expires = timedelta(minutes=settings.ACCESS_TOKEN_EXPIRE_MINUTES)
    return {
        "access_token": security.create_access_token(
            user.id, expires_delta=access_token_expires
        ),
        "token_type": "bearer",
    }


@router.post("/login/test-token", response_model=schemas.User)
def test_token(current_user: models.User = Depends(deps.get_current_user)) -> Any:
    """
    Test access token
    """
    return current_user


@router.post("/password-recovery/{email}", response_model=schemas.Msg)
def recover_password(email: str, db: Session = Depends(deps.get_db)) -> Any:
    """
    Password Recovery
    """
    user = crud.user.get_by_email(db, email=email)

    if not user:
        raise HTTPException(
            status_code=404,
            detail="The user with this username does not exist in the system.",
        )
    password_reset_token = generate_password_reset_token(email=email)
    send_reset_password_email(
        email_to=user.email, email=email, token=password_reset_token
    )
    return {"msg": "Password recovery email sent"}


@router.post("/reset-password/", response_model=schemas.Msg)
def reset_password(
    token: str = Body(...),
    new_password: str = Body(...),
    db: Session = Depends(deps.get_db),
) -> Any:
    """
    Reset password
    """
    email = verify_password_reset_token(token)
    if not email:
        raise HTTPException(status_code=400, detail="Invalid token")
    user = crud.user.get_by_email(db, email=email)
    if not user:
        raise HTTPException(
            status_code=404,
            detail="The user with this username does not exist in the system.",
        )
    elif not crud.user.is_active(user):
        raise HTTPException(status_code=400, detail="Inactive user")
    hashed_password = get_password_hash(new_password)
    user.hashed_password = hashed_password
    db.add(user)
    db.commit()
    return {"msg": "Password updated successfully"}


@router.post("/phone_status/", response_model=schemas.Msg)
def phone_status(
    telephone: str = Body(...),
    db: Session = Depends(deps.get_db),
) -> Any:
    """
    Check phone status
    """
    if not telephone_validate(telephone):
        raise HTTPException(status_code=400, detail="Invalid telephone number")

    user = crud.user.get_by_telephone(db, telephone=telephone)
    if user:
        raise HTTPException(
            status_code=400,
            detail="Phone number is busy",
        )
    return {"msg": "Free telephone number"}



from typing import List

from fastapi.encoders import jsonable_encoder
from sqlalchemy.orm import Session

from jwtserver.crud.base import CRUDBase
from jwtserver.models.item import Item
from jwtserver.schemas.item import ItemCreate, ItemUpdate


class CRUDItem(CRUDBase[Item, ItemCreate, ItemUpdate]):
    def create_with_owner(
        self, db: Session, *, obj_in: ItemCreate, owner_id: int
    ) -> Item:
        obj_in_data = jsonable_encoder(obj_in)
        db_obj = self.model(**obj_in_data, owner_id=owner_id)
        db.add(db_obj)
        db.commit()
        db.refresh(db_obj)
        return db_obj

    def get_multi_by_owner(
        self, db: Session, *, owner_id: int, skip: int = 0, limit: int = 100
    ) -> List[Item]:
        return (
            db.query(self.model)
            .filter(Item.owner_id == owner_id)
            .offset(skip)
            .limit(limit)
            .all()
        )


item = CRUDItem(Item)



from typing import TYPE_CHECKING

from sqlalchemy import Boolean, Column, Integer, String
from sqlalchemy.orm import relationship

from jwtserver.db.base_class import Base

if TYPE_CHECKING:
    from .item import Item  # noqa: F401


class User(Base):
    id = Column(Integer, primary_key=True, index=True)
    full_name = Column(String, index=True)
    email = Column(String, unique=True, index=True, nullable=False)
    telephone = Column(String, unique=True, index=True, nullable=False)
    hashed_password = Column(String, nullable=False)
    is_active = Column(Boolean(), default=True)
    is_superuser = Column(Boolean(), default=False)
    items = relationship("Item", back_populates="owner")



# Import all the models, so that Base has them before being
# imported by Alembic
from jwtserver.db.base_class import Base  # noqa
from jwtserver.models.item import Item  # noqa
from jwtserver.models.user import User  # noqa



from fastapi import APIRouter

from jwtserver.api.api_v1.endpoints import items, login, users, utils

api_router = APIRouter()
api_router.include_router(login.router, tags=["login"])
api_router.include_router(users.router, prefix="/users", tags=["users"])
api_router.include_router(utils.router, prefix="/utils", tags=["utils"])
api_router.include_router(items.router, prefix="/items", tags=["items"])



from pydantic import BaseModel


class CheckCodeResponseModel(BaseModel):
    reg_token: str


class PhoneStatusModel(BaseModel):
    free: bool
    telephone: str


class RespError(BaseModel):
    error: str
    block_time: int | None


class RespSendCodeModel(BaseModel):
    send: bool
    block_time: float
    method: str


class Data(BaseModel):
    telephone: str
    password: str
    reg_token: str


class AccessTokenResponseModel(BaseModel):
    access_token: str
    token_type: str



import logging
from datetime import datetime, timedelta
from pathlib import Path
from typing import Any, Dict, Optional

import emails
import phonenumbers
from emails.template import JinjaTemplate
from jose import jwt

from jwtserver.core.config import settings


def send_email(
    email_to: str,
    subject_template: str = "",
    html_template: str = "",
    environment: Dict[str, Any] = {},
) -> None:
    assert settings.EMAILS_ENABLED, "no provided configuration for email variables"
    message = emails.Message(
        subject=JinjaTemplate(subject_template),
        html=JinjaTemplate(html_template),
        mail_from=(settings.EMAILS_FROM_NAME, settings.EMAILS_FROM_EMAIL),
    )
    smtp_options = {"host": settings.SMTP_HOST, "port": settings.SMTP_PORT}
    if settings.SMTP_TLS:
        smtp_options["tls"] = True
    if settings.SMTP_USER:
        smtp_options["user"] = settings.SMTP_USER
    if settings.SMTP_PASSWORD:
        smtp_options["password"] = settings.SMTP_PASSWORD
    response = message.send(to=email_to, render=environment, smtp=smtp_options)
    logging.info(f"send email result: {response}")


def send_test_email(email_to: str) -> None:
    project_name = settings.PROJECT_NAME
    subject = f"{project_name} - Test email"
    with open(Path(settings.EMAIL_TEMPLATES_DIR) / "test_email.html") as f:
        template_str = f.read()
    send_email(
        email_to=email_to,
        subject_template=subject,
        html_template=template_str,
        environment={"project_name": settings.PROJECT_NAME, "email": email_to},
    )


def send_reset_password_email(email_to: str, email: str, token: str) -> None:
    project_name = settings.PROJECT_NAME
    subject = f"{project_name} - Password recovery for user {email}"
    with open(Path(settings.EMAIL_TEMPLATES_DIR) / "reset_password.html") as f:
        template_str = f.read()
    server_host = settings.SERVER_HOST
    link = f"{server_host}/reset-password?token={token}"
    send_email(
        email_to=email_to,
        subject_template=subject,
        html_template=template_str,
        environment={
            "project_name": settings.PROJECT_NAME,
            "username": email,
            "email": email_to,
            "valid_hours": settings.EMAIL_RESET_TOKEN_EXPIRE_HOURS,
            "link": link,
        },
    )


def send_new_account_email(email_to: str, username: str, password: str) -> None:
    project_name = settings.PROJECT_NAME
    subject = f"{project_name} - New account for user {username}"
    with open(Path(settings.EMAIL_TEMPLATES_DIR) / "new_account.html") as f:
        template_str = f.read()
    link = settings.SERVER_HOST
    send_email(
        email_to=email_to,
        subject_template=subject,
        html_template=template_str,
        environment={
            "project_name": settings.PROJECT_NAME,
            "username": username,
            "password": password,
            "email": email_to,
            "link": link,
        },
    )


def generate_password_reset_token(email: str) -> str:
    delta = timedelta(hours=settings.EMAIL_RESET_TOKEN_EXPIRE_HOURS)
    now = datetime.utcnow()
    expires = now + delta
    exp = expires.timestamp()
    encoded_jwt = jwt.encode(
        {"exp": exp, "nbf": now, "sub": email},
        settings.SECRET_KEY,
        algorithm="HS256",
    )
    return encoded_jwt


def verify_password_reset_token(token: str) -> Optional[str]:
    try:
        decoded_token = jwt.decode(token, settings.SECRET_KEY, algorithms=["HS256"])
        return decoded_token["email"]
    except jwt.JWTError:
        return None


def telephone_validate(telephone) -> bool:
    return phonenumbers.is_valid_number(phonenumbers.parse(telephone, None))



from typing import Optional, Dict, Any

from pydantic import BaseSettings, BaseModel, RedisDsn, PostgresDsn, validator, EmailStr

from jwtserver.utils.SMSC import SMSModel


class TokenModel(BaseModel):
    base_sol: str = "1234567890987654321"
    secret_key: str = None
    algorithm: str = "HS256"
    # 60 minutes * 24 hours * 8 days = 7 days
    ACCESS_TOKEN_EXPIRE_MINUTES: int = 60 * 24 * 7
    REFRESH_EXPIRE_TIME: int = 10800  # minutes


class PostgresConfig(BaseModel):
    # password: str = "example"
    # user: str = "postgres"
    # db: str = "postgres"
    # server: str = "localhost"
    # port: int = 5432
    dsn: Optional[PostgresDsn] = None
    # f"postgresql+psycopg2://{user}:{password}@{server}:{port}/{db}"

    @validator("dsn", pre=True)
    def assemble_db_connection(cls, v: Optional[str], values: Dict[str, Any]) -> Any:
        if isinstance(v, str):
            return v

        return PostgresDsn.build(
            scheme="postgresql+psycopg2",
            user=values.get("user"),
            password=values.get("password"),
            host=values.get("server"),
            port=values.get("port"),
            path=f"/{values.get('db') or ''}",
        )


class RedisConfig(BaseModel):
    redis_dsn: RedisDsn = None
    max_connections: int = 10


class RecaptchaModel(BaseModel):
    secret_key: str = "uudud"
    score: float = 0.7
    for_tests: bool = False


class GoogleConfig(BaseModel):
    recaptcha: RecaptchaModel = RecaptchaModel()


class ServerConfig(BaseModel):
    # domain: Set[str] = set()
    host: str = "0.0.0.0"
    port: int = 8000
    max_requests: int = 1000
    debug: bool = True


class SMTPConfig(BaseModel):
    tls: bool = True
    port: Optional[int] = None
    host: Optional[str] = None
    user: Optional[str] = None
    password: Optional[str] = None


class EmailsConfig(BaseModel):
    from_email: Optional[EmailStr] = None
    from_name: Optional[str] = None

    @validator("from_name")
    def get_project_name(cls, v: Optional[str], values: Dict[str, Any]) -> str:
        if not v:
            return values["PROJECT_NAME"]
        return v

    EMAIL_RESET_TOKEN_EXPIRE_HOURS: int = 48
    EMAIL_TEMPLATES_DIR: str = "/app/app/email-templates/build"
    EMAILS_ENABLED: bool = False

    @validator("EMAILS_ENABLED", pre=True)
    def get_emails_enabled(cls, v: bool, values: Dict[str, Any]) -> bool:
        return bool(
            values.get("SMTP_HOST")
            and values.get("SMTP_PORT")
            and values.get("EMAILS_FROM_EMAIL")
        )


class Settings(BaseSettings):
    environment: str = "production"
    server: ServerConfig = ServerConfig()
    token: TokenModel = TokenModel()
    postgres: PostgresConfig = PostgresConfig()
    smtp: SMTPConfig = SMTPConfig()
    redis: RedisConfig = RedisConfig()
    google: GoogleConfig = GoogleConfig()
    sms: SMSModel = SMSModel()
    API_V1_STR: str = "/api/v1"
    recaptcha: bool = False

    class Config:
        env_file = ".env"
        env_nested_delimiter = "__"


settings = Settings()



from typing import Optional

from pydantic import BaseModel


# Shared properties
class ItemBase(BaseModel):
    title: Optional[str] = None
    description: Optional[str] = None


# Properties to receive on item creation
class ItemCreate(ItemBase):
    title: str


# Properties to receive on item update
class ItemUpdate(ItemBase):
    pass


# Properties shared by models stored in DB
class ItemInDBBase(ItemBase):
    id: int
    title: str
    owner_id: int

    class Config:
        orm_mode = True


# Properties to return to client
class Item(ItemInDBBase):
    pass


# Properties properties stored in DB
class ItemInDB(ItemInDBBase):
    pass



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['jwtserver',
 'jwtserver.api',
 'jwtserver.api.add_later',
 'jwtserver.api.api_v1',
 'jwtserver.api.api_v1.endpoints',
 'jwtserver.core',
 'jwtserver.crud',
 'jwtserver.db',
 'jwtserver.middlewares',
 'jwtserver.migration',
 'jwtserver.migration.versions',
 'jwtserver.models',
 'jwtserver.old_dependencies',
 'jwtserver.recaptcha',
 'jwtserver.schemas',
 'jwtserver.tests',
 'jwtserver.utils']

package_data = \
{'': ['*'], 'jwtserver': ['email-templates/build/*', 'email-templates/src/*']}

install_requires = \
['alembic>=1.10.2,<2.0.0',
 'asyncpg==0.27.0',
 'celery>=5.2.7,<6.0.0',
 'emails>=0.6,<0.7',
 'fastapi>=0.95.0,<0.96.0',
 'httpx>=0.23.3,<0.24.0',
 'loguru>=0.6.0,<0.7.0',
 'passlib==1.7.4',
 'phonenumbers>=8.13.7,<8.14.0',
 'psycopg2==2.9.5',
 'pydantic[email]>=1.10.6,<2.0.0',
 'pytest==7.2.2',
 'python-dotenv>=1.0.0,<2.0.0',
 'python-jose[cryptography]>=3.3.0,<4.0.0',
 'python-multipart>=0.0.6,<0.0.7',
 'redis[hiredis]>=4.5.1,<5.0.0',
 'setuptools>=67.6.0,<67.7.0',
 'sqlalchemy>=2.0.7,<3.0.0',
 'sqlalchemy_utils>=0.40.0,<0.41.0',
 'starlette>=0.26.1,<0.27.0',
 'uvicorn>=0.21.1,<0.22.0']

entry_points = \
{'console_scripts': ['upgradehead = jwtserver.commands:db_upgrade_cmd']}

setup_kwargs = {
    'name': 'jwtserver',
    'version': '0.0.39',
    'description': 'jwt authorization server',
    'long_description': '# JWT server\n\n_JWTServer лёгкий и быстрый микросервис JWT._\n\n[![Package version](https://img.shields.io/pypi/v/jwtserver?color=%2334D058&label=pypi%20package)](https://pypi.org/project/jwtserver)\n[![Supported Python versions](https://img.shields.io/pypi/pyversions/jwtserver.svg?color=%2334D058)](https://pypi.org/project/jwtserver)\n\nJWT Server является микросервисом для авторизации пользователей. Имеющий гибкие настройки и разные версии API.\n\n* Подключение Google Recaptcha V3\n* Отправка кода через сервис <a href="https://smsc.ru/" target="_blank">https://smsc.ru/</a> (**стадия тестирования**)\n\n## Особенности\n\n* Быстрый старт\n* Походит для тестирования frontend\n* Основан на Fast API framework и вдохновлен работами <a href="https://github.com/tiangolo" target="_blank">tiangolo</a>\n\n## Route Entrypoints\n\n### Login\n* **[POST]** - /api/v1/login/access-token\n* **[POST]** - /api/v1/login/test-token\n* **[POST]** - /api/v1/password-recovery/{email}\n* **[POST]** - /api/v1/reset-password/\n* **[POST]** - /api/v1/phone_status/\n\n### Users\n* **[GET]** - /api/v1/users/\n* **[POST]** - /api/v1/users/\n* **[GET]** - /api/v1/users/me\n* **[PUT]** - /api/v1/users/me\n* **[POST]** - /api/v1/users/open\n* **[GET]** - /api/v1/users/{user_id}\n* **[PUT]** - /api/v1/users/{user_id}\n\n### Utils\n* **[POST]** - /api/v1/utils/test-celery/\n* **[POST]** - /api/v1/utils/test-email/\n\n### Items\n* **[GET]** - /api/v1/items/\n* **[POST]** - /api/v1/items/\n* **[GET]** - /api/v1/items/{id}\n* **[PUT]** - /api/v1/items/{id}\n* **[DELETE]** - /api/v1/items/{id}\n\n---\n\n**Поддержка кода** <a href="https://github.com/goltsevnet/jwtserver" target="_blank">https://github.com/goltsevnet/jwtserver</a>\n\n---\n\n## Зависимости\n\n* **uvicorn** <a href="https://www.uvicorn.org/" target="_blank" class="external-link">https://www.uvicorn.org/</a>\n* **fastapi** <a href="https://fastapi.tiangolo.com/" target="_blank" class="external-link">https://fastapi.tiangolo.com/</a>\n* **starlette** <a href="https://www.starlette.io/" target="_blank" class="external-link">https://www.starlette.io/</a>\n* **passlib** <a href="https://pypi.org/project/passlib/" target="_blank" class="external-link">https://pypi.org/project/passlib/</a>\n* **pydantic** <a href="https://pydantic-docs.helpmanual.io/" target="_blank" class="external-link">https://pydantic-docs.helpmanual.io/</a>\n* **redis** <a href="https://pypi.org/project/redis/" target="_blank" class="external-link">https://pypi.org/project/redis/</a>\n* **python-jose** <a href="https://pypi.org/project/python-jose/" target="_blank" class="external-link">https://pypi.org/project/python-jose/</a>\n* **sqlalchemy** <a href="https://pypi.org/project/SQLAlchemy/" target="_blank" class="external-link">https://pypi.org/project/SQLAlchemy/</a>\n* **sqlalchemy_utils** <a href="https://sqlalchemy-utils.readthedocs.io/" target="_blank" class="external-link">https://sqlalchemy-utils.readthedocs.io/</a>\n* **asyncpg** <a href="https://pypi.org/project/asyncpg/" target="_blank" class="external-link">https://pypi.org/project/asyncpg/</a>\n* **psycopg2-binary** <a href="https://pypi.org/project/psycopg2-binary/" target="_blank" class="external-link">https://pypi.org/project/psycopg2-binary/</a>\n* **httpx** <a href="https://www.python-httpx.org/" target="_blank" class="external-link">https://www.python-httpx.org/</a>\n* **python-dotenv** <a href="https://pypi.org/project/python-dotenv/" target="_blank" class="external-link">https://pypi.org/project/python-dotenv/</a>\n\n## Установка\n\n```shell\npython -m pip install jwtserver \n```\n\nПодготовьте базу данных `Postgres 14` и выше версии.\n\nИспользуйте как вариант `docker-compose.yaml`\n```dockerfile\nversion: \'3.1\'\n\nservices:\n  db:\n    image: postgres:14.1-alpine\n    restart: always\n    build: .\n    environment:\n      POSTGRES_PASSWORD: example\n    ports:\n      - 5433:5432\n```\n\nОбязательно добавьте переменную в среду: `POSTGRES__DSN` для миграций.\n\nВыполните команду:\n```shell\nupgradehead\n```\n\nДолжны увидеть:\n```shell\nINFO  [alembic.runtime.migration] Context impl PostgresqlImpl.\nINFO  [alembic.runtime.migration] Will assume transactional DDL.\nINFO  [alembic.runtime.migration] Running upgrade  -> 98f3d19d2c0d, initial\n```\n\nОстальные переменные можно добавить в файл `.env` или через dotenv.\n\n```dotenv\nENVIRONMENT=production\n\nSMS__LOGIN=login\nSMS__PASSWORD=password\nSMS__IGNORE_ATTEMPTS=True\n\nPOSTGRES__DSN=postgresql+psycopg2://postgres:example@localhost:5433/postgres\n\nRECAPTCHA=True\n\nGOOGLE__RECAPTCHA__SECRET_KEY=SECRET_KEY\nTOKEN__V1_SOL=12345679100987654321\nTOKEN__SECRET_KEY=SECRET_KEY\nTOKEN__ACCESS_TOKEN_EXPIRE_MINUTES=90\nAPI_V1_STR=/api/v1\n```\n\n## Примеры:\n\n### Для разработки\n\n* создайте файл `dev.py`\n\n```python\nfrom jwtserver.server import dev\n\nif __name__ == "__main__":\n    dev(host="localhost", port=5000, log_level="info")\n```\n\n\n### Интерактивная API документация\n\nоткройте _Interactive API docs_ <a href="http://localhost:5000/docs" target="_blank" class="external-link">http://localhost:5000/docs</a>\n\nВы увидите автоматическую интерактивную документацию по API.\n\n### Альтернативная API документация\n\nоткройте _Alternative  API redoc_ <a href="http://localhost:5000/redoc" target="_blank" class="external-link">http://localhost:5000/redoc</a>\n\n### Для продукции\n\n* создайте файл `main.py`\n\n```python\nfrom jwtserver.app import app\n\napp.debug = False\n```\n\n* **Gunicorn:** `gunicorn main:app`\n\n## Лицензия\nЭтот проект находится под лицензией Apache 2.0.\n',
    'author': 'goltsevnet',
    'author_email': 'mark@goltsev.net',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8.1,<4.0.0',
}


setup(**setup_kwargs)



from __future__ import annotations

from fastapi import FastAPI
from httpx import AsyncClient
from redis import Redis

telephone_for_test = "+79138915678"


# def check_refresh_token_cookie(cookies):
#     for cookie in cookies:
#         if cookie.name == 'refresh_token':
#             assert cookie.has_nonstandard_attr('HttpOnly')
#             expires = datetime.fromtimestamp(cookie.expires)
#             expires_fix = timedelta(minutes=1) + expires
#             delta_days = (expires_fix - datetime.now()).days
#             # assert delta_days == KEYS.REFRESH_TOKEN_EXPIRE_DAYS
#             return True
#     return False


# async def test_phone_status(client: AsyncClient, app: FastAPI, redis_client: Redis):
#     """
#     Проверка на свободность номера и ничего больше
#     """
#
#     data = {
#         "telephone": telephone_for_test,
#         "recaptcha_token": "success:SignUpPage/PhoneStatus:0.8",
#     }
#
#     response = await client.post(app.url_path_for("reg:phone_status"), json=data)
#
#     assert response.status_code == 200, response.text
#     assert response.headers["content-type"] == "application/json"
#
#     # телефон должен быть свободный
#     assert response.json()["free"]
#     assert response.json()["telephone"]
#
#     # проверяем, что ничего нет в редисе
#     assert not redis_client.hgetall(telephone_for_test)


# async def test_send_code(client: AsyncClient, app: FastAPI):
#     """
#     Отправка кода
#     """
#
#     response = await client.post(
#         app.url_path_for("reg:send_code"), json=telephone_for_test
#     )
#
#     assert response.status_code == 200, response.text
#     assert response.json()["send"]
#
#     repeat_response = await client.post(
#         app.url_path_for("reg:send_code"), json=telephone_for_test
#     )
#
#     assert repeat_response.status_code == 200, repeat_response.text
#     assert repeat_response.json()["send"]
#     assert repeat_response.json()["method"] == "call"
#     assert type(repeat_response.json()["block_time"]) == float


#
# def test_check_code_valid():
#     data = {"telephone": telephone_for_test, "code": redis.get(telephone_for_test)}
#     headers = {"accept": "application/x-www-form-urlencoded"}
#     response = client.post(
#         "/api/v1/auth/check_code",
#         headers=headers,
#         data=data
#     )
#     assert response.status_code == 200
#     assert response.json()['status'] == 'valid'
#
#
# def test_check_code_fake():
#     """
#     Кидаем на существующий номер фейковый код
#     """
#     data = {"telephone": telephone_for_test, "code": "99999"}
#     headers = {"accept": "application/x-www-form-urlencoded"}
#     response = client.post(
#         "/api/v1/auth/check_code",
#         headers=headers,
#         data=data
#     )
#     assert response.status_code == 200
#     assert response.json()['status'] == 'not valid'
#
#
# def test_registration_user():
#     response = client.post(
#         "/api/v1/auth/registration_user",
#         data={
#             "telephone": telephone_for_test,
#             "password": registration_password,
#             "code": redis.get(telephone_for_test),
#             "fingerprint": fingerprint
#         }
#     )
#     response_json = response.json()
#     assert response.status_code == 200
#     assert 'password' not in response_json
#     assert response_json['token_type'] == 'bearer'
#     access_token = response_json['access_token']
#     payload = access_token.split('.', 2)[1] + "==="
#     payload_decoded = json.loads(base64.urlsafe_b64decode(payload))
#     assert 'uuid' and 'isActive' and 'exp' in payload_decoded
#     assert payload_decoded['isActive']
#     assert UUID(payload_decoded['uuid']).version == 4
#     global registration_uuid
#     registration_uuid = payload_decoded['uuid']
#     refresh_token = response.cookies['refresh_token']
#     global registration_access_token
#     registration_access_token = access_token
#     assert refresh_token
#     assert check_refresh_token_cookie(response.cookies)
#     refresh_token_data = redis.hgetall(refresh_token)
#     assert refresh_token_data['uuid']
#     assert refresh_token_data['refresh_token']
#     assert refresh_token_data['fingerprint']
#     assert refresh_token_data['expires_date']
#     assert refresh_token_data['create_date']
#     assert float(refresh_token_data['expires_date']) > datetime.now().timestamp()
#     assert float(refresh_token_data['create_date']) < datetime.now().timestamp()
#
#
# def test_registration_user_fake_code():
#     response = client.post(
#         "/api/v1/auth/registration_user",
#         headers={
#             "accept": "application/x-www-form-urlencoded"
#         },
#         data={
#             "telephone": telephone_for_test,
#             "password": registration_password,
#             "code": "1234",
#             "fingerprint": fingerprint_fake
#         }
#     )
#     response_json = response.json()
#     assert response.status_code == 400
#     assert response_json['detail'] == 'Fake user'
#
#
# def test_get_token():
#     response = client.post(
#         "/api/v1/auth/token",
#         headers={
#             "accept": "application/x-www-form-urlencoded"
#         },
#         data={
#             'uuid': registration_uuid,
#             'password': registration_password
#         }
#     )
#     resp_json = response.json()
#     assert 'access_token' and 'token_type' in resp_json
#     assert response.status_code == 200, response.text
#     payload = jwt.decode(resp_json['access_token'], KEYS.SECRET_KEY, algorithms=[KEYS.ALGORITHM])
#     assert "uuid" and "exp" in payload
#     assert payload['uuid'] == registration_uuid, payload
#     assert type(payload['exp']) == float, payload['exp']
#     assert resp_json
#
#
# def test_get_token_for_fake_password():
#     data_copy = data_fake_auth.copy()
#     data_copy.update({'uuid': 'abdula'})
#     response = client.post(
#         "/api/v1/auth/token",
#         headers={
#             "accept": "application/x-www-form-urlencoded"
#         },
#         data=data_copy
#     )
#
#     assert response.status_code == 401, response.text
#     assert response.json()['detail'] == 'Incorrect uuid or password'
#
#
# def test_get_token_for_fake_user():
#     data_copy = data_fake_auth.copy()
#     data_copy.update({'password': '2131'})
#     response = client.post(
#         "/api/v1/auth/token",
#         headers={
#             "accept": "application/x-www-form-urlencoded"
#         },
#         data=data_copy
#     )
#
#     assert response.status_code == 401, response.text
#     assert response.json()['detail'] == 'Incorrect uuid or password'
#
#
# def client_post_refresh_token(**kwargs):
#     return client.post(
#         "/api/v1/auth/refresh_token",
#         data={"access_token": registration_access_token, "fingerprint": fingerprint},
#         **kwargs
#     )
#
#
# def test_new_refresh_token():
#     response = client_post_refresh_token()
#     assert response.status_code == 200, response.text
#     assert "access_token" and "token_type" in response.json()
#     assert check_refresh_token_cookie(response.cookies)
#     assert len(response.json()['access_token'].split('.')) == 3
#     assert response.json()['token_type'] == 'bearer'
#
#     global registration_access_token
#     registration_access_token = response.json()['access_token']
#
#
# def test_repeat_new_refresh_token():
#     response = client_post_refresh_token()
#     assert response.json()['access_token'] != registration_access_token
#     assert response.status_code == 200, response.text
#     assert "access_token" and "token_type" in response.json()
#     assert check_refresh_token_cookie(response.cookies)
#     assert len(response.json()['access_token'].split('.')) == 3
#     assert response.json()['token_type'] == 'bearer'
#
#
# def test_new_refresh_token__old_is_none():
#     client.cookies.clear()
#     response = client_post_refresh_token()
#     assert response.status_code == 422, response.text
#     response_json = response.json()
#     assert response_json['detail']
#
#
# def test_new_refresh_token__old_is_fake():
#     response = client_post_refresh_token(cookies={"refresh_token": "fake_token"})
#     assert response.status_code == 400, response.text
#     response_json = response.json()
#     assert response_json['detail']



from .crud_item import item
from .crud_user import user

# For a new basic set of CRUD operations you could just do

# from .base import CRUDBase
# from app.models.item import Item
# from app.schemas.item import ItemCreate, ItemUpdate

# item = CRUDBase[Item, ItemCreate, ItemUpdate](Item)



from jwtserver.database import AsyncSessionLocal


# async def async_db_session():
#     """Databases pool fabric connection, auto close connection"""
#     async with AsyncSessionLocal() as session:
#         yield session
async def async_db_session():
    """Databases pool fabric connection, auto close connection"""
    try:
        db = AsyncSessionLocal()
        yield db
    finally:
        db.close()



import logging
import sys

from fastapi import FastAPI, Depends
from loguru import logger
from starlette.middleware.base import BaseHTTPMiddleware
from starlette.middleware.cors import CORSMiddleware

from jwtserver.api.api_v1.api import api_router
from jwtserver.api.deps import recaptcha_google_v3_action
from jwtserver.core.config import settings

__all__ = ["app", "create_app"]

from jwtserver.middlewares.stock import middleware_stock

origins = [
    "http://localhost",
    "http://localhost:8080",
    "http://localhost:8000",
    "http://localhost:5000",
    "http://localhost:3000",
]
description = """[Full JWT Server docs](https://jwtserver.markgoltsev.net)"""
tags_metadata = [
    {
        "name": "login",
        "externalDocs": {
            "description": "Login external docs",
            "url": "https://jwtserver.markgoltsev.net/en/api_v1/login",
        },
    },
    {
        "name": "users",
        "externalDocs": {
            "description": "users external docs",
            "url": "https://jwtserver.markgoltsev.net/en/api_v1/users",
        },
    },
    {
        "name": "utils",
        "externalDocs": {
            "description": "utils external docs",
            "url": "https://jwtserver.markgoltsev.net/en/api_v1/users",
        },
    },
    {
        "name": "items",
        "externalDocs": {
            "description": "items external docs",
            "url": "https://jwtserver.markgoltsev.net/en/api_v1/users",
        },
    },
]


def create_app(_title="JWT server", lvl_logging="INFO") -> FastAPI:
    dependencies = []

    if settings.recaptcha:
        dependencies.append(Depends(recaptcha_google_v3_action))

    _app = FastAPI(
        title=_title,
        description=description,
        version="0.0.39",
        license_info={
            "name": "Apache 2.0",
            "url": "https://www.apache.org/licenses/LICENSE-2.0.html",
        },
        openapi_tags=tags_metadata,
    )

    _app.add_middleware(
        CORSMiddleware,
        allow_origins=origins,
        allow_credentials=True,
        allow_methods=["*"],
        allow_headers=["*"],
    )

    if settings.recaptcha:
        _app.add_middleware(BaseHTTPMiddleware, dispatch=middleware_stock)

    _app.include_router(
        api_router,
        prefix=settings.API_V1_STR,
        dependencies=dependencies,
    )

    # _app.dependency_overrides = {
    #     get_settings: lambda: _config,
    # }
    _app.debug = True

    def enable_logger(sink=sys.stderr, level="DEBUG"):
        logging.basicConfig(level=logging.DEBUG)
        logger.configure(handlers=[{"sink": sink, "level": level}])
        logger.enable("aria2p")

    enable_logger(level=lvl_logging)
    return _app


app = create_app()



"""JWTServer is a lightweight and fast JWT microservice."""
# __version__ = "0.0.19"

# from jwtserver.app import app as app
# from jwtserver.app import create_app as create_app
# from jwtserver.server import dev as dev
